/*
  Communicate with Micro Lambda Wireless, MLXX Synthesizers, using the Arduino UNO serial monitor.
  For wire harness diagram see - MLXX synthesizer harness for Arduino interface 36 inch.xls
  D.S. 6/12/16
*/
const int clockPin = 8;//Serial Clock line
const int dataInPin = 10; //Serial Data (IN) line RX
const int dataOutPin = 11; //Serial Data (OUT)line TX
const int selectPin = 12;//Serial Select line
const int busyPin = 13; //Serial Busy line
//Pin 14 = Serial Logic Ground
String bitOrder = "MSBFIRST"; //serial stream bit order
int busy;
char sendArray[16];
char recvArray[16];
int counter;
int b = 0;
int i = 0;
int x = 0;

void setup() {
  //set pins to output/input serial data
  pinMode(clockPin, OUTPUT);
  pinMode(dataInPin, INPUT);
  pinMode(dataOutPin, OUTPUT);
  pinMode(selectPin, OUTPUT);
  pinMode(busyPin, INPUT);
  digitalWrite(selectPin, HIGH); //Init the selectPin high
  Serial.begin(9600);
  while (! Serial); // Wait untilSerial is ready - For Leonardo
  Serial.println("MLXX Synthesizer Interface - Enter a Command (ASCII)");
}

void loop() {
  while (Serial.available() > 0) {
    //read all char's to send from serial monitor
    delay(1); //wait for serial port
    sendArray[i] = Serial.read();
    i = i + 1;
    delay(1); //wait for serial port
    //when all char's received, call serial send frequency routine (sends ASCII #'s)
    if (Serial.available() == 0) {
      sendFreq();
    }
    //prints the command sent / received to the serial monitor window
    if (Serial.available() == 0) {
      printComm();
    }
  }
}

//send / receive command routine
void sendFreq() {
  do {
    busy = digitalRead(busyPin); //read busy line, if low continue
  } while (busy == 1);

  //loop for array length of chars to send
  for (counter = 0; counter != i; counter++)
  {
    digitalWrite(selectPin, LOW); //selectPin Low and hold low for as long as you are transmitting
    delayMicroseconds(10);
    //shift out 8 bits with xx uS delay (Bit Bang)
    for (b = 0; b < 8; b++)  {
      if (bitOrder == LSBFIRST)
      { digitalWrite(dataOutPin, !!(sendArray[counter] & (1 << b)));
        delayMicroseconds(2);
      }
      else
      { digitalWrite(dataOutPin, !!(sendArray[counter] & (1 << (7 - b))));
        delayMicroseconds(2);
      }
      //toggle clock line
      digitalWrite(clockPin, HIGH);
      delayMicroseconds(2);
      digitalWrite(clockPin, LOW);
      delayMicroseconds(2);
    }
    delayMicroseconds(5); //delay between chars for byte visibility on Dig. scope
  }
  digitalWrite(selectPin, HIGH); //pull the selectPin to save the data
  digitalWrite(dataOutPin, LOW); //set data out low when done
  digitalWrite(clockPin, LOW); //set clock low when done

  delayMicroseconds(10); //wait for a xuS between write/read

  //read back data from unit if available
  do {
    busy = digitalRead(busyPin); //read busy line, if low continue
    } while (busy == 1);
  
  digitalWrite(clockPin, HIGH); //clock line high to start for read
   
  //read back data from unit if available, 16 bytes
  for (counter = 0; counter < 16; counter++)
  {
    digitalWrite(selectPin, LOW); //selectPin Low and hold low for as long as you are receiving
    delayMicroseconds(10);
    //shift in 8 bits with xx uS delay (Bit Bang)
    for (b = 0; b < 8; b++)  {
      if (bitOrder == LSBFIRST)
      { recvArray[counter] |= digitalRead(dataInPin) << b; //shift in bits
        delayMicroseconds(2);
      }
      else
      { recvArray[counter] |= digitalRead(dataInPin) << (7 - b); //shift in bits MSB first
        delayMicroseconds(2);
      }
      //toggle clock line
      digitalWrite(clockPin, LOW);
      delayMicroseconds(2);
      digitalWrite(clockPin, HIGH);
      delayMicroseconds(2);
    }
    delayMicroseconds(5); //delay between chars for byte visibility on Dig. scope
    //exit if char = NUL
    if (recvArray[counter] == 0)
      { counter = 16;
      }
  }
  digitalWrite(selectPin, HIGH); //pull the selectPin to save the data
  digitalWrite(dataOutPin, LOW); //set data out low when done
  digitalWrite(clockPin, LOW); //set clock low when done
  
  delayMicroseconds(10); //wait for a 5uS between commands
}

//format and print command sent/received to serial monitor window
void printComm() {
  Serial.print("Command Sent = ");
  for (x = 0; x < 16; x++) {
    Serial.print(sendArray[x]);
  }
  Serial.println();
  Serial.print("Info Received = ");
  for (x = 0; x < 16; x++) {
    Serial.print(recvArray[x]);
  }

  Serial.println();
  //clear var.
  i = 0;
  x = 0;
  b = 0;
  memset(&sendArray[0], 0, sizeof(sendArray)); //clear array
  memset(&recvArray[0], 0, sizeof(recvArray)); //clear array
}
